/*
 * Copyright 2006-2007  Anthony Rowe and Adam Goode
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  Created on: 02.03.2010
 *      Author:  
 *       
 */

#include <stdio.h>
#include "dvr.h"


void dvr_system_setup (void)
{

  // --- enable and connect the PLL (Phase Locked Loop) ---
  // a. set multiplier and divider
  PLLCFG = MSEL | (1<<PSEL1) | (0<<PSEL0);
  // b. enable PLL
  PLLCON = (1<<PLLE);
  // c. feed sequence
  PLLFEED = PLL_FEED1;
  PLLFEED = PLL_FEED2;
  // d. wait for PLL lock (PLOCK bit is set if locked)
  while (!(PLLSTAT & (1<<PLOCK)));
  // e. connect (and enable) PLL
  PLLCON = (1<<PLLE) | (1<<PLLC);
  // f. feed sequence
  PLLFEED = PLL_FEED1;
  PLLFEED = PLL_FEED2;

  // --- setup and enable the MAM (Memory Accelerator Module) ---
  // a. start change by turning of the MAM (redundant)
  MAMCR = 0;
  // b. set MAM-Fetch cycle to 3 cclk as recommended for >40MHz
  MAMTIM = MAM_FETCH;
  // c. enable MAM
  MAMCR = MAM_MODE;

  // --- set VPB speed ---
  VPBDIV = VPBDIV_VAL;

  // --- map INT-vector ---
  MEMMAP = MEMMAP_USER_FLASH_MODE;

  //  Fast GPIO
  SCS = 0x1;

//  PCONP = 0xFFFFFFFF;
}

/* init_ms_timer() -     1    
 *      
 */
void init_ms_timer(void) {
	T2TCR = 0;   // turn off timer
	T2MCR = 0;    // disable interrupt
	T2TC = 0;    // clear counter
	T2PC = 0;    // clear prescale count
	T2PR  = (uint16_t)((FOSC*PLL_M/VPBDIV_VAL)/1000);  // every 1 ms
	T2TCR = 1;   // start the timer
}

void wait_ms (uint16_t delay)
{
  T2TCR = 0x2;
  T2TCR = 0x2;
  T2TCR = 0x2;
  T2TCR = 0x2;
  T2TCR = 0x1;
  while (T2TC < delay);
}

uint8_t send_cam_cmd(uint8_t addr, uint8_t data) {
	uint8_t i2c_data[2];
	i2c_data[0] = addr;
	i2c_data[1] = data;
	return i2cMasterSendNI(CAM_I2C_ADDR, 2, i2c_data);
}

uint8_t camera_init(void) {
	uint8_t i2c_result;
	i2c_result = I2C_OK;

	i2c_result |= send_cam_cmd(0x11, BIT(7) | 0x5); // Enable Digital PLL option and PCLK = XVCLK/6 = ~2.3
	i2c_result |= send_cam_cmd(0x12, BIT(3)); // QQCIF,     QCIF,  ""
	i2c_result |= send_cam_cmd(0x39, 0x43); // OFON,      ,  ""
	i2c_result |= send_cam_cmd(0x04, 0x01<<2); //    ,  ""
	i2c_result |= send_cam_cmd(0x1e, BIT(5));    // MVFP,  ,   ,       ,  ""

//       ...
//     ,       ...   

	i2c_result |= send_cam_cmd(0x6b, 0x0a);    // DBLV       band gap reference adjustment
	i2c_result |= send_cam_cmd(0x6a, 0x41);    // 		   LSB of Banding Filter (effective only when COM11[0] is high)
	i2c_result |= send_cam_cmd(0x3b, 0x09);    // COM11      use half frame and manual banding filter mode
	i2c_result |= send_cam_cmd(0x13, 0xe0);    // COM8       enable fast AGC/AEC, enable AEC, and enable Banding filter
	i2c_result |= send_cam_cmd(0x01, 0x80);	 // BLUE	   Blue channel gain setting, default 0x80
	i2c_result |= send_cam_cmd(0x02, 0x80);	 // RED		   Red channel gain setting, default 0x80
	i2c_result |= send_cam_cmd(0x00, 0x00);	 // GAIN	   AGC, range 0x00 to 0xFF
	i2c_result |= send_cam_cmd(0x10, 0x00);    // AECH       exposure value set default 0x40
	i2c_result |= send_cam_cmd(0x13, 0xe5);    // COM8       now enable AGC/AEC
	i2c_result |= send_cam_cmd(0x39, 0x43);    // OPON       ???
	i2c_result |= send_cam_cmd(0x38, 0x12);    // ACOM       ???
	i2c_result |= send_cam_cmd(0x37, 0x00);	 // ADC		   ???
	i2c_result |= send_cam_cmd(0x35, 0x91);    // RSVD	   ???
	i2c_result |= send_cam_cmd(0x0e, 0x20);    // COM5       ???
	//i2c_result |= send_cam_cmd(0x1e, 0x30);    // MVFP       Mirror/VFlip enable, mirror and vflip enable for self-portrait (hardware OV9650_BO)
	//i2c_result |= send_cam_cmd(0x1e, 0x00);
	//i2c_result |= send_cam_cmd(0x04, 0x00);    // COM1       no line skip option
	i2c_result |= send_cam_cmd(0x0c, 0x00);    // COM3       set to default
	i2c_result |= send_cam_cmd(0x0d, 0x00);	 // COM4	   default to 0x00

	i2c_result |= send_cam_cmd(0x14, 0x1a);    // COM9       automatic gain control ceilings
	//i2c_result |= send_cam_cmd(0x15, 0x10);    // COM10      set up pclk, vsync and href signals
	i2c_result |= send_cam_cmd(0x3f, 0xa6);	 // EDGE	   what edge enhancement ???

	i2c_result |= send_cam_cmd(0x41, 0x02);	 // COM16	   Color matrix coefficient double option enable ???
	i2c_result |= send_cam_cmd(0x42, 0xc1);	 // COM17	   edge enhancement option, tri-state output etc
	i2c_result |= send_cam_cmd(0x1b, 0x00);    // PSHFT      set pclk delay from href

	i2c_result |= send_cam_cmd(0x33, 0xe2);    // CHLF       array current control
	i2c_result |= send_cam_cmd(0x34, 0xbf);    // ARBLM      array reference control

	i2c_result |= send_cam_cmd(0x96, 0x04);	 // RSVD	   again, reserved register
	i2c_result |= send_cam_cmd(0x3a, 0x0c); 	 // TSLB	   line buffer test option for UV output and sequence setting

	i2c_result |= send_cam_cmd(0x8e, 0x00);	 // COM24	   Reserved register
	i2c_result |= send_cam_cmd(0x3c, 0x77);    // COM12      href options

	i2c_result |= send_cam_cmd(0x8b,0x06);
	i2c_result |= send_cam_cmd(0x94,0x88);
	i2c_result |= send_cam_cmd(0x95,0x88);
	i2c_result |= send_cam_cmd(0x40,0xc1);
	i2c_result |= send_cam_cmd(0x29,0x3f);
	i2c_result |= send_cam_cmd(0x0f,0x42);
	i2c_result |= send_cam_cmd(0x3d,0x92);
	i2c_result |= send_cam_cmd(0x69,0x40);	 // Manual Banding Filter MSB
	i2c_result |= send_cam_cmd(0x5c,0xb9);	 // REG[0x59-0x61] all RSVD
	i2c_result |= send_cam_cmd(0x5d,0x96);
	i2c_result |= send_cam_cmd(0x5e,0x10);
	i2c_result |= send_cam_cmd(0x59,0xc0);
	i2c_result |= send_cam_cmd(0x5a,0xaf);
	i2c_result |= send_cam_cmd(0x5f,0xe0);
	i2c_result |= send_cam_cmd(0x60,0x8c);
	i2c_result |= send_cam_cmd(0x61,0x20);

	i2c_result |= send_cam_cmd(0x8d,0x02);	 // color bar test mode disable, digital AWB enable

	i2c_result |= send_cam_cmd(0x13,0xe7);	 // AGC, AWB, AEC all enabled here

	i2c_result |= send_cam_cmd(0x4f,0x3a);	 // REG[4F-58] are Matrix coefficients
	i2c_result |= send_cam_cmd(0x50,0x3d);
	i2c_result |= send_cam_cmd(0x51,0x03);
	i2c_result |= send_cam_cmd(0x52,0x12);
	i2c_result |= send_cam_cmd(0x53,0x26);
	i2c_result |= send_cam_cmd(0x54,0x38);
	i2c_result |= send_cam_cmd(0x55,0x40);
	i2c_result |= send_cam_cmd(0x56,0x40);
	i2c_result |= send_cam_cmd(0x57,0x40);
	i2c_result |= send_cam_cmd(0x58,0x0d);

// TODO:    ,          
	i2c_result |= send_cam_cmd(0x8c,0x23);	 // De-noise enable, white-pixel erase enable
	i2c_result |= send_cam_cmd(0x3e,0x02);

	i2c_result |= send_cam_cmd(0x8f,0xdf);	 // Digital BLC offset sign???

	i2c_result |= send_cam_cmd(0x90,0x00);	 // Digital BLC B channel offset value set default
	i2c_result |= send_cam_cmd(0x91,0x00);	 // Digital BLC R channel offset value set default
	i2c_result |= send_cam_cmd(0x9f,0x00);	 // Digital BLC Gb channel offset value set default
	i2c_result |= send_cam_cmd(0xa0,0x00);	 // Digital BLC Gr channel offset value set default

	i2c_result |= send_cam_cmd(0x24,0x98);	 // AEW stable operating region upper limit
	i2c_result |= send_cam_cmd(0x25,0x88);	 // AEW stable operating region lower limit
	i2c_result |= send_cam_cmd(0x26,0xd3);	 // AGC/AEC fast mode operating region
	i2c_result |= send_cam_cmd(0x2a,0x00);	 // Horizontal dummy pixel insert MSB
	i2c_result |= send_cam_cmd(0x2b,0x00);	 // Horizontal dummy insert LSB
	i2c_result |= send_cam_cmd(0x2d,0x00);	 // Vertical LSB of dummy lines

	i2c_result |= send_cam_cmd(0x6c,0x40);	 // REG[6C-7B] are Gamma Curve setup GSP
	i2c_result |= send_cam_cmd(0x6d,0x30);
	i2c_result |= send_cam_cmd(0x6e,0x4b);
	i2c_result |= send_cam_cmd(0x6f,0x60);
	i2c_result |= send_cam_cmd(0x70,0x70);
	i2c_result |= send_cam_cmd(0x71,0x70);
	i2c_result |= send_cam_cmd(0x72,0x70);
	i2c_result |= send_cam_cmd(0x73,0x70);
	i2c_result |= send_cam_cmd(0x74,0x60);
	i2c_result |= send_cam_cmd(0x75,0x60);
	i2c_result |= send_cam_cmd(0x76,0x50);
	i2c_result |= send_cam_cmd(0x77,0x48);
	i2c_result |= send_cam_cmd(0x78,0x3a);
	i2c_result |= send_cam_cmd(0x79,0x2e);
	i2c_result |= send_cam_cmd(0x7a,0x28);
	i2c_result |= send_cam_cmd(0x7b,0x22);

	i2c_result |= send_cam_cmd(0x7c,0x04);	 // REG[7C-8A] are Gamma Curve setup GST
	i2c_result |= send_cam_cmd(0x7d,0x07);
	i2c_result |= send_cam_cmd(0x7e,0x10);
	i2c_result |= send_cam_cmd(0x7f,0x28);
	i2c_result |= send_cam_cmd(0x80,0x36);
	i2c_result |= send_cam_cmd(0x81,0x44);
	i2c_result |= send_cam_cmd(0x82,0x52);
	i2c_result |= send_cam_cmd(0x83,0x60);
	i2c_result |= send_cam_cmd(0x84,0x6c);
	i2c_result |= send_cam_cmd(0x85,0x78);
	i2c_result |= send_cam_cmd(0x86,0x8c);
	i2c_result |= send_cam_cmd(0x87,0x9e);
	i2c_result |= send_cam_cmd(0x88,0xbb);
	i2c_result |= send_cam_cmd(0x89,0xd2);
	i2c_result |= send_cam_cmd(0x8a,0xe6);

	return i2c_result;
}

void debug_get_cam_registers_dump(void) {
	for (uint8_t i = 0; i <= 0xAA; i++) {
		uint8_t i2c_data;
		i2cMasterSendNI(CAM_I2C_ADDR, 1, &i);
		i2cMasterReceiveNI(CAM_I2C_ADDR, 1, &i2c_data);
		uart0Putch(i2c_data);
	}
	return;
}

void default_irq (void) {
	EXTINT = 1;
	uart0Puts("interrupt !\n");
	VICVectAddr = 0;
}

void FIQ_Routine (void)  {
	return;
}

void SWI_Routine (void)  {
	return;
}

void UNDEF_Routine (void) {
	return;
}
