/*
 * OV9650_LPC2103_example
 *
 *     OV9650 (TGA130V10)    QQCIF     UART  (3 , GRB).
 *     ,  ,       ,    
 *        :)
 *
 *      bmp-   :) BMP-  ,
 *    (  )    .
 *    ,     0x36,   
 *     GRB,   RGB.         88x72
 *     "24-  bmp".  WinHEX'    0x36 raw-,
 *    (   ,     FF FF FF ... ( ),
 *     ), ,  ,    ;)
 *
 *  Created on: 28.03.2010
 *      Author:  
 *   roboforum.ru
 */

#include <dvr.h>

#define CLIP(x) {if(x<0) x=0;if(x>255) x=255;}


void sample_image(void);
void transfer_column_RGB(const uint32_t *column, uint32_t num);
void transfer_column_YUV(const uint32_t *column, uint32_t num);

uint32_t column[800];

int main(void) {
	dvr_system_setup();

	uart0Init(UART_BAUD(DEBUG_UART_BAUD), UART_8N1, UART_FIFO_8); // setup the UART
	init_ms_timer();

	FIODIR = (1<<DEBUG_LED_PIN);
	FIOCLR = (1<<DEBUG_LED_PIN);

	sample_image();

	while(1);
	return 0;
}

void sample_image(void) {
	// I2C: 
	i2cInit(1000, 1000);

	//   CAM_RESET   
	FIODIR = FIODIR | (1<<CAM_RESET_PIN);
	//   RESET  LOW
	FIOCLR = (1<<CAM_RESET_PIN);
	wait_ms(10);

	//    XVCLK   ~14.75
	PINSEL1 = (PINSEL1 & ~0xC00) | 0x800; //   CAM_XVCLK  
	T3TCR=0;   // turn off timer
	T3TC=0;    // clear counter
	T3PC=0;    // clear prescale count
	T3PR = 0;  //   c  PCLK
	T3PWMCON = (1<<CAM_XVCLK_MR)|(1<<3); // 
	T3MCR = 1<<10;
	T3MR0 = 2; //    
	T3MR3 = 4; //  duty 50%
	T3TCR=1;   // start the timer
	// 
	wait_ms(5);
	//   RESET  HIGH
	FIOSET = (1<<CAM_RESET_PIN);
	// 
	wait_ms(10);
	//   RESET  LOW
	FIOCLR = (1<<CAM_RESET_PIN);
	// 
	wait_ms(10);

	// I2C:  
	uint8_t i2c_result;
	i2c_result = camera_init();
	if (i2c_result == I2C_ERROR_NODEV) {
		uart0Puts(",    !\n");
		return;
	}

	debug_uart0Puts("  !\n");

	// 
	wait_ms(1000);


	//         uart
	// .. uart  ,        
	//    ,        .
	//   ,        :)
	//     ,      ,  .
	//   Blanking HREF  ,     -   ,
	//    ,    PCLK   .
	//
	debug_uart0Puts(" \n");

	int height = 0;
	int width = 0;
	//      
	int frame_line = 0;
	while (frame_line < IMG_HEIGHT) {
		//      VSYNC
		while (~FIOPIN & (1<<CAM_VSYNC_PIN));
		while (FIOPIN & (1<<CAM_VSYNC_PIN));
		height = 0;
		while (height < IMG_HEIGHT) {
			//    HREF
			while (~FIOPIN & (1<<CAM_HREF_PIN));
				//     PCLK
				while (width < IMG_WIDTH) {
/*
					// 1- : X-R4-R3-R2-R1-R0-G4-G3
					while (~FIOPIN & (1<<CAM_PCLK_PIN));
					column[2*width] = FIOPIN;
					while (FIOPIN & (1<<CAM_PCLK_PIN));
					// 2- : G2-G1-G0-B4-B3-B2-B1-B0
					while (~FIOPIN & (1<<CAM_PCLK_PIN));
					column[2*width+1] = FIOPIN;
					while (FIOPIN & (1<<CAM_PCLK_PIN));
*/
					while (~FIOPIN & (1<<CAM_PCLK_PIN));
					column[4*width] = FIOPIN;
					while (FIOPIN & (1<<CAM_PCLK_PIN));
					while (~FIOPIN & (1<<CAM_PCLK_PIN));
					column[4*width+1] = FIOPIN;
					while (FIOPIN & (1<<CAM_PCLK_PIN));
					while (~FIOPIN & (1<<CAM_PCLK_PIN));
					column[4*width+2] = FIOPIN;
					while (FIOPIN & (1<<CAM_PCLK_PIN));
					while (~FIOPIN & (1<<CAM_PCLK_PIN));
					column[4*width+3] = FIOPIN;
					while (FIOPIN & (1<<CAM_PCLK_PIN));

					width++;
				}
				width = 0;
			if (height == frame_line) { //    
				break;
			}
			height++;
			//    HREF
			while (FIOPIN & (1<<CAM_HREF_PIN));
		}
		//    UART
//		transfer_column_RGB(column, IMG_WIDTH);
		transfer_column_YUV(column, IMG_WIDTH);
		frame_line++;
	}

	debug_uart0Puts(" \n");

	return;
}

void transfer_column_RGB(const uint32_t *column, uint32_t num) {
	int width = 0;
	for (width=0; width < num; width++) {
		uint32_t temp1, temp2;
		uint8_t ch_red, ch_green, ch_blue, green_channel;
		temp1 = column[width*2];
		temp2 = column[width*2+1];
		// 1- : X-R4-R3-R2-R1-R0-G4-G3
		ch_red = ((temp1>>(17-4)) & 0x10) | ((temp1>>8) & 0x0F);
		green_channel = (temp1>>(6-3)) & 0x18;
		// 2- : G2-G1-G0-B4-B3-B2-B1-B0
		ch_green = green_channel | ((temp2>>(17-1)) & 0x06) | ((temp2>>11) & 0x01);
		ch_blue = (temp2>>6) & 0x1F;
		uart0Putch(ch_blue*8);
		uart0Putch(ch_green*8);
		uart0Putch(ch_red*8);
	}
	return;
}

void transfer_column_YUV(const uint32_t *column, uint32_t num) {
	int width = 0;
	for (width=0; width < num; width++) {
		uint32_t temp0, temp1, temp2, temp3, R, G, B;
		int Y1, Y2, Y, U, V, C, D, E;
		temp0 = column[width*4];
		temp1 = column[width*4+1];
		temp2 = column[width*4+2];
		temp3 = column[width*4+3];

		U = ((temp0 >> (17-6)) & 0xC0) | ((temp0 >> 6) & 0x3F);
		Y1 = ((temp1 >> (17-6)) & 0xC0) | ((temp1 >> 6) & 0x3F);
		V = ((temp2 >> (17-6)) & 0xC0) | ((temp2 >> 6) & 0x3F);
		Y2 = ((temp3 >> (17-6)) & 0xC0) | ((temp3 >> 6) & 0x3F);

		Y = (Y1+Y2) >> 1;

		//  YUV  RGB,  : http://msdn.microsoft.com/en-us/library/ms893078.aspx
		C = Y - 16;
		D = U - 128;
		E = V - 128;
		R = (298*C + 409 * E + 128) >> 8;
		G = (298*C - 100 * D - 208 * E + 128) >> 8;
		B = (298*C + 516 * D + 128) >> 8;

		CLIP(R);
		CLIP(G);
		CLIP(B);
		uart0Putch(B);
		uart0Putch(G);
		uart0Putch(R);
	}
	return;
}
